@@ -30,47 +30,45 @@ from odcs.common.types import (
     COMPOSE_RESULTS, COMPOSE_FLAGS, INVERSE_PUNGI_SOURCE_TYPE_NAMES)
 
 
-def _load_allowed_clients_attrs(key, attrs):
+def _set_default_client_allowed_attrs(ret_attrs, attrs):
     """
-    Loads attributes from the
-    conf.allowed_clients[key][user_name/group_name] dict. If the requested
-    attribute is not found in the loaded dict, the conf.allowed_$attr_name
-    is used as a default.
+    Helper method adding default allowed_client_attrs into `ret_attrs`.
 
-    :param str key: "users" or "groups".
-    :param list attrs: List of attribute names to load from the dict.
-    :return: Dict with loaded attributes.
+    If some requested attributes from `attrs` are missing in `ret_attrs` list,
+    try to get them from "conf.allowed_$attr_name". If they are not there
+    too, use empty list to disallow everything.
     """
-    clients = conf.allowed_clients.get(key, {})
-    ret_attrs = []
-    if key == "users":
-        # Check if the user is defined in clients, if not, return None.
-        if flask.g.user.username in clients:
-            ret_attrs = dict(copy.deepcopy(clients[flask.g.user.username]))
-        else:
-            return None
-    elif key == "groups":
-        # Check if the group is defined in clients, if not, return None
-        for group in flask.g.groups:
-            if group in clients:
-                ret_attrs = dict(copy.deepcopy(clients[group]))
-                break
-        else:
-            return None
-    else:
-        raise ValueError(
-            "Unknown key %r passed to _load_allowed_clients_attrs" % key)
-
-    # If some requested attributes are missing in allowed_clients variable,
-    # try to get them from "conf.gallowed_$attr_name". If they are not there
-    # too, use empty list to disallow everything.
     for attr in attrs:
         if attr not in ret_attrs:
             ret_attrs[attr] = getattr(conf, "allowed_%s" % attr, [])
-
     return ret_attrs
 
 
+def _load_allowed_clients_attr(attrs):
+    """
+    Loads attributes from the conf.allowed_clients dict based on the logged
+    in user and its groups. If the requested attribute is not found in
+    the loaded dict, the conf.allowed_$attr_name is used as a default.
+
+    :param list attrs: List of attribute names to load from the dict.
+    :return: Generator of Dicts with loaded attributes.
+    """
+    # Check if logged user exists in "users" definition of allowed_clients.
+    clients = conf.allowed_clients.get("users", {})
+    if flask.g.user.username in clients:
+        ret_attrs = dict(copy.deepcopy(clients[flask.g.user.username]))
+        ret_attrs = _set_default_client_allowed_attrs(ret_attrs, attrs)
+        yield ret_attrs
+    else:
+        # Check if group is defined in allowed_clients "groups".
+        clients = conf.allowed_clients.get("groups", {})
+        for group in flask.g.groups:
+            if group in clients:
+                ret_attrs = dict(copy.deepcopy(clients[group]))
+                ret_attrs = _set_default_client_allowed_attrs(ret_attrs, attrs)
+                yield ret_attrs
+
+
 def _enum_int_to_str_list(enum_dict, val):
     """
     Convenient method converting int value to list of strings
@@ -97,6 +95,7 @@ def raise_if_input_not_allowed(**kwargs):
       - source_types - compose.source_type
       - sources - compose.source
       - arches - compose.arches
+      - compose_types - compose.compose_type
 
     The decision whether the user is allowed or not is done based on
     conf.allowed_clients value.
@@ -104,42 +103,47 @@ def raise_if_input_not_allowed(**kwargs):
     if conf.auth_backend == 'noauth':
         return
 
-    # Prefer args for particular user - these overrides group ones.
-    attrs = _load_allowed_clients_attrs("users", kwargs.keys())
-    if not attrs:
-        attrs = _load_allowed_clients_attrs("groups", kwargs.keys())
-        if not attrs:
-            raise Forbidden("User %s not allowed to operate with any "
-                            "compose" % flask.g.user.username)
+    errors = set()
+    for attrs in _load_allowed_clients_attr(kwargs.keys()):
+        found_error = False
+        for name, values in kwargs.items():
+            if name not in attrs:
+                # This should not happen, but be defensive in this part of code...
+                errors.add(
+                    "User %s not allowed to operate with compose with %s=%r."
+                    % (flask.g.user.username, name, values))
+                continue
 
-    for name, values in kwargs.items():
-        if name not in attrs:
-            # This should not happen, but be defensive in this part of code...
-            raise Forbidden(
-                "User %s not allowed to operate with compose with %s=%r"
-                % (flask.g.user.username, name, values))
+            # Convert integers from db format to string list.
+            if name == "source_types":
+                values = INVERSE_PUNGI_SOURCE_TYPE_NAMES[values]
+            elif name == "flags":
+                values = _enum_int_to_str_list(COMPOSE_FLAGS, values)
+            elif name == "results":
+                values = _enum_int_to_str_list(COMPOSE_RESULTS, values)
 
-        # Conver integers from db format to string list.
-        if name == "source_types":
-            values = INVERSE_PUNGI_SOURCE_TYPE_NAMES[values]
-        elif name == "flags":
-            values = _enum_int_to_str_list(COMPOSE_FLAGS, values)
-        elif name == "results":
-            values = _enum_int_to_str_list(COMPOSE_RESULTS, values)
+            if type(values) == int:
+                values = [values]
+            elif isinstance(values, six.string_types):
+                # `arches` and `sources` are white-space separated lists.
+                values = values.split(" ")
 
-        if type(values) == int:
-            values = [values]
-        elif isinstance(values, six.string_types):
-            # `arches` and `sources` are white-space separated lists.
-            values = values.split(" ")
-
-        for value in values:
-            allowed_values = attrs[name]
-            if ((not allowed_values or value not in allowed_values) and
-                    allowed_values != [""]):
-                raise Forbidden(
-                    "User %s not allowed to operate with compose with %s=%s"
-                    % (flask.g.user.username, name, value))
+            for value in values:
+                allowed_values = attrs[name]
+                if ((not allowed_values or value not in allowed_values) and
+                        allowed_values != [""]):
+                    errors.add(
+                        "User %s not allowed to operate with compose with %s=%s."
+                        % (flask.g.user.username, name, value))
+                    found_error = True
+                    break
+        if not found_error:
+            return
+    if errors:
+        raise Forbidden(" ".join(list(errors)))
+    else:
+        raise Forbidden(
+            "User %s not allowed to operate with any compose." % flask.g.user.username)
 
 
 def validate_json_data(dict_or_list, level=0, last_dict_key=None):
@@ -170,7 +174,8 @@ def validate_json_data(dict_or_list, level=0, last_dict_key=None):
             # not exploitable.
             if last_dict_key in ["packages"]:
                 continue
-            allowed_chars = [' ', '-', '/', '_', '.', ':', '#']
+            allowed_chars = [' ', '-', '/', '_', '.', ':', '#', '+', '?', '$',
+                             '~']
             if not all(c.isalnum() or c in allowed_chars for c in v):
                 raise ValueError(
                     "Only alphanumerical characters and %r characters "
@@ -269,7 +274,8 @@ def filter_composes(flask_request):
     """
     search_query = dict()
 
-    for key in ['owner', 'source_type', 'source', 'state', 'koji_task_id']:
+    for key in ['owner', 'source_type', 'source', 'state', 'koji_task_id',
+                'pungi_compose_id', 'compose_type', 'label']:
         if flask_request.args.get(key, None):
             search_query[key] = flask_request.args[key]
 
