@@ -122,57 +122,25 @@ class Config(object):
             'type': str,
             'default': "pungi-koji",
             'desc': 'Name or full-path to pungi-koji binary.'},
+        'pungi_config_validate': {
+            'type': str,
+            'default': "",
+            'desc': 'Name or full-path to pungi-config-validate binary. '
+                    'If set to empty string, no validation is done.'},
         'pungi_timeout': {
             'type': int,
             'default': 3600,
             'desc': 'Time in seconds after which the local pungi-koji is '
                     'killed and compose is marked as failed'},
-        'pungi_runroot_enabled': {
-            'type': bool,
-            'default': False,
-            'desc': 'When True, ODCS will run Pungi in Koji runroot'},
-        'pungi_parent_runroot_channel': {
-            'type': str,
-            'default': "",
-            'desc': 'Name of Koji channel to run pungi runroot task with.'},
-        'pungi_parent_runroot_packages': {
-            'type': list,
-            'default': "",
-            'desc': 'List of packages to install to pungi Koji runroot task.'},
-        'pungi_parent_runroot_mounts': {
-            'type': list,
-            'default': "",
-            'desc': 'List of mounts to mount in pungi Koji runroot task.'},
-        'pungi_parent_runroot_weight': {
-            'type': float,
-            'default': 3,
-            'desc': 'Weight of pungi Koji runroot task.'},
-        'pungi_parent_runroot_tag': {
-            'type': str,
-            'default': "",
-            'desc': 'Name of tag to run pungi Koji runroot task for.'},
-        'pungi_parent_runroot_arch': {
-            'type': str,
-            'default': "x86_64",
-            'desc': 'Architecture to run pungi Koji runroot task for.'},
-        'pungi_runroot_target_dir': {
-            'type': str,
-            'default': "",
-            'desc': 'Path to target dir to which store composes done by pungi in runroot'},
-        'pungi_runroot_target_dir_url': {
-            'type': str,
-            'default': "",
-            'desc': 'Public facing URL to target_dir.'},
+        'mergerepo_timeout': {
+            'type': int,
+            'default': 1800,
+            'desc': 'Time in seconds after which the mergerepo_c is '
+                    'killed and compose is marked as failed'},
         'pungi_conf_path': {
             'type': str,
             'default': "/etc/odcs/pungi.conf",
             'desc': 'Full path to the pungi.conf jinja2 template.'},
-        'pungi_runroot_koji_conf_path': {
-            'type': str,
-            'default': "/etc/odcs/runroot_koji.conf",
-            'desc': 'Full path to the runroot_koji.conf configuration file. '
-                    'This file holds Koji configuration used in a Pungi parent '
-                    'runroot task if pungi_runroot_enabled is True.'},
         'target_dir': {
             'type': str,
             'default': "/tmp",
@@ -199,7 +167,7 @@ class Config(object):
             'desc': 'Number of concurrent Pungi processes.'},
         'allowed_source_types': {
             'type': list,
-            'default': ["tag", "module"],
+            'default': ["tag", "module", "build"],
             'desc': 'Allowed source types.'},
         'allowed_flags': {
             'type': list,
@@ -217,6 +185,10 @@ class Config(object):
             'type': list,
             'default': [""],
             'desc': 'Allowed sources.'},
+        'allowed_compose_types': {
+            'type': list,
+            'default': [""],
+            'desc': 'Allowed compose types.'},
         'auth_ldap_server': {
             'type': str,
             'default': '',
@@ -246,10 +218,15 @@ class Config(object):
             'type': list,
             'default': [],
             'desc': 'Required scopes for submitting request to run new compose.'},
+        'base_module_names': {
+            'type': set,
+            'default': set(['platform', 'bootstrap']),
+            'desc': ("Set of module names which defines the product version "
+                     "(by their stream) of modules depending on them.")},
         'messaging_backend': {
             'type': str,
             'default': '',
-            'desc': 'Messaging backend, fedmsg or umb.'},
+            'desc': 'Messaging backend, rhmsg or fedora-messaging.'},
         'messaging_broker_urls': {
             'type': list,
             'default': [],
@@ -321,6 +298,12 @@ class Config(object):
             'type': str,
             'default': None,
             'desc': 'Kerberos principal to use for Koji auth.'},
+        'koji_tag_cache_cleanup_timeout': {
+            'type': int,
+            'default': 30,
+            'desc': 'Number of days after which the cached Koji tag data '
+                    'stored in the "koji_tag_cache" directory will be '
+                    'removed.'},
         'raw_config_urls': {
             'type': dict,
             'default': {},
@@ -335,6 +318,75 @@ class Config(object):
                     'file. This file holds Pungi configuration which should '
                     'import real pungi configuration from raw_config.conf '
                     'in order to override some variables.'},
+        'raw_config_schema_override': {
+            'type': str,
+            'default': "",
+            'desc': 'Full path to the JSON file defining Pungi config schema '
+                    'override. This file is passed to "pungi-config-validate" '
+                    'using the --schema-override option and can influence '
+                    'the raw_config validation.'},
+        'pungi_koji_args': {
+            'type': list,
+            'default': [],
+            'desc': 'Command line arguments used to construct pungi-koji '
+                    'command.'
+        },
+        'raw_config_pungi_koji_args': {
+            'type': dict,
+            'default': {},
+            'desc': 'Command line argument for raw_config source type, which '
+                    'overwrite arguments listed PUNGI_KOJI_ARGS.'
+        },
+        'celery_config': {
+            'type': dict,
+            'default': {},
+            'desc': 'Configuration dict to pass to Celery.'
+        },
+        'celery_broker_url': {
+            'type': str,
+            'default': "",
+            'desc': 'Celery broker URL'
+        },
+        'celery_pulp_composes_queue': {
+            'type': str,
+            'default': "pulp_composes",
+            'desc': 'Name of the Celery queue for Pulp composes.'
+        },
+        'celery_pungi_composes_queue': {
+            'type': str,
+            'default': "pungi_composes",
+            'desc': 'Name of the Celery queue for Pungi composes.'
+        },
+        'celery_cleanup_queue': {
+            'type': str,
+            'default': "cleanup",
+            'desc': 'Name of the Celery queue for cleanup task.'
+        },
+        'celery_router_config': {
+            'type': dict,
+            'default': {
+                "routing_rules": {
+                    "odcs.server.celery_tasks.generate_pungi_compose": {
+                        "pungi_composes": {
+                            "source_type": 3,
+                        },
+                    },
+                    "odcs.server.celery_tasks.generate_pulp_compose": {
+                        "pulp_composes": {
+                            "source_type": 4,
+                        },
+                    },
+                },
+                "cleanup_task": "odcs.server.celery_tasks.run_cleanup",
+                "default_queue": "pungi_composes",
+            },
+            'desc': 'Configuration for custom celery router.'
+        },
+        'runroot_extra_mounts': {
+            'type': list,
+            'default': [],
+            'desc': 'Extra mountpoint directories for odcs-mock-runroot.'
+        },
     }
 
     def __init__(self, conf_section_obj):
@@ -464,3 +516,18 @@ class Config(object):
         if not os.path.isfile(s):
             raise ValueError("Pungi config template doesn't exist: %s" % s)
         self._pungi_conf_path = s
+
+    def _setifok_celery_router_config(self, celery_router_config):
+        if type(celery_router_config) != dict:
+            raise TypeError("celery_router_config must be a dict.")
+
+        required_config_keys = ["routing_rules", "cleanup_task", "default_queue"]
+
+        for conf_key in required_config_keys:
+            if not celery_router_config.get(conf_key):
+                raise KeyError("celery_router_config is missing %s" % conf_key)
+
+        if type(celery_router_config["routing_rules"]) != dict:
+            raise TypeError("routing_rules must be a dict.")
+
+        self._celery_router_config = celery_router_config
