@@ -27,11 +27,12 @@ from xml.etree import ElementTree
 from concurrent.futures import ThreadPoolExecutor
 from datetime import timedelta
 from six.moves.urllib.parse import urlparse
+from distutils.spawn import find_executable
 
 from flufl.lock import Lock
 
 from odcs.server import log, conf
-from odcs.server.utils import makedirs, find_executable, execute_cmd
+from odcs.server.utils import makedirs, execute_cmd
 
 
 class MergeRepo(object):
@@ -46,17 +47,21 @@ class MergeRepo(object):
         :param str path: Path to store the file to.
         :param str url: URL of file to download.
         :rtype: str
-        :return: content of downloaded file.
+        :return: path to the downloaded file
         """
         log.info("%r: Downloading %s", self.compose, url)
-        r = requests.get(url)
+        r = requests.get(url, timeout=conf.net_timeout, stream=True)
         r.raise_for_status()
 
         filename = os.path.basename(url)
         makedirs(os.path.join(path, "repodata"))
-        with open(os.path.join(path, "repodata", filename), "wb") as f:
-            f.write(r.content)
-        return r.content
+        outfile = os.path.join(path, "repodata", filename)
+        with open(outfile, "wb") as f:
+            for chunk in r.iter_content(chunk_size=None):
+                f.write(chunk)
+        response_len = os.stat(outfile).st_size
+        log.info("%r: Downloaded %d bytes from %s", self.compose, response_len, url)
+        return outfile
 
     def _download_repodata(self, path, baseurl):
         """
@@ -76,7 +81,9 @@ class MergeRepo(object):
 
         # Download the repomd.xml.
         repomd_url = os.path.join(baseurl, "repodata", "repomd.xml")
-        repomd = self._download_file(path, repomd_url)
+        repomd_path = self._download_file(path, repomd_url)
+        with open(repomd_path) as f:
+            repomd = f.read()
         tree = ElementTree.fromstring(repomd)
 
         # In case the repomd.xml did not change since the last compose, use
@@ -105,15 +112,17 @@ class MergeRepo(object):
                 data_url = os.path.join(baseurl, data_location)
                 downloader.submit(self._download_file, path, data_url)
 
-    def run(self, arch, repos):
+    def run(self, arch, repos, repo_name):
         """
         Merges multiple RPM repositories and stores the output to
-        `os.path.join(compose.result_repo_dir, arch)`.
+        `os.path.join(compose.result_repo_dir, repo_name, arch)`.
 
         Raises an RuntimeError in case of error.
 
         :param str arch: Architecture of RPMs in repositories.
         :param list repos: List of URLs pointing to repos to merge.
+        :param str repo_name: Name of the repository defining the subdirectory
+            in `compose.result_repo_dir`.
         """
         # Multiple MergeRepo tasks can be running in the same time, so the
         # pulp_repo_cache must be protected by lock.
@@ -123,6 +132,12 @@ class MergeRepo(object):
         # Contains paths to per pulp repo pulp_repo_cache sub-directories.
         repo_paths = []
 
+        # Remove duplicated URLs from repos. It is useless to merge the same URLs and it would
+        # also break the locking code which tries to lock the same cache directory twice.
+        # The mergerepo_c can handle the case when we end up with just single repo in the `repos`.
+        repos = list(set(repos))
+        repos.sort()
+
         parsed_url = urlparse(repos[0])
         repo_prefix = "%s://%s" % (parsed_url.scheme, parsed_url.hostname)
         repo_prefix = repo_prefix.strip("/") + "/"
@@ -149,7 +164,7 @@ class MergeRepo(object):
             if not mergerepo_exe:
                 raise RuntimeError("mergerepo_c is not available on system")
 
-            result_repo_dir = os.path.join(self.compose.result_repo_dir, arch)
+            result_repo_dir = os.path.join(self.compose.result_repo_dir, repo_name, arch)
             makedirs(result_repo_dir)
 
             args = [mergerepo_exe, "--method", "nvr", "-o",
@@ -160,7 +175,7 @@ class MergeRepo(object):
                 args.append("-r")
                 args.append(repo)
 
-            execute_cmd(args)
+            execute_cmd(args, timeout=conf.mergerepo_timeout)
         finally:
             for lock in locks:
                 if lock.is_locked:
