@@ -32,13 +32,22 @@ from odcs.server.errors import NotFound
 from odcs.server.models import Compose
 from odcs.common.types import (
     COMPOSE_RESULTS, COMPOSE_FLAGS, COMPOSE_STATES, PUNGI_SOURCE_TYPE_NAMES,
-    PungiSourceType)
+    PungiSourceType, MULTILIB_METHODS)
 from odcs.server.api_utils import (
     pagination_metadata, filter_composes, validate_json_data,
     raise_if_input_not_allowed)
 from odcs.server.auth import requires_role, login_required
 from odcs.server.auth import require_scopes
 
+try:
+    from odcs.server.celery_tasks import (
+        generate_pulp_compose, generate_pungi_compose)
+    CELERY_AVAILABLE = True
+except ImportError:
+    log.exception(
+        "Cannot import celery_tasks. The Celery support is turned off.")
+    CELERY_AVAILABLE = False
+
 
 api_v1 = {
     'composes': {
@@ -84,19 +93,23 @@ api_v1 = {
 class ODCSAPI(MethodView):
     def _get_compose_owner(self):
         if conf.auth_backend == "noauth":
-            log.warn("Cannot determine the owner of compose, because "
-                     "'noauth' auth_backend is used.")
+            log.warning(
+                "Cannot determine the owner of compose, because "
+                "'noauth' auth_backend is used.")
             return "unknown"
         else:
             return g.user.username
 
     def _get_seconds_to_live(self, request_data):
-        if "seconds-to-live" in request_data:
+        seconds_to_live = request_data.get("seconds_to_live")
+        # Fallback to old name of this variable to keep backward compatibility.
+        if seconds_to_live is None:
+            seconds_to_live = request_data.get("seconds-to-live")
+        if seconds_to_live:
             try:
-                return min(int(request_data['seconds-to-live']),
-                           conf.max_seconds_to_live)
+                return min(int(seconds_to_live), conf.max_seconds_to_live)
             except ValueError:
-                err = 'Invalid seconds-to-live specified in request: %s' % \
+                err = 'Invalid seconds_to_live specified in request: %s' % \
                     request_data
                 log.error(err)
                 raise ValueError(err)
@@ -104,6 +117,30 @@ class ODCSAPI(MethodView):
             return conf.seconds_to_live
 
     def get(self, id):
+        """ Returns ODCS composes.
+
+        If ``id`` is set, only the ODCS compose defined by that ID is
+        returned.
+
+        :query string owner: Return only composes owned by this:ref:`owner<owner>`.
+        :query number source_type: Return only composes of this :ref:`source type<source_type>`.
+        :query string source: Return only composes built from this :ref:`source<source>`.
+        :query number state: Return only composes built from this :ref:`state<state>`.
+        :query string order_by: Order the composes by the given field. If ``-`` prefix is used,
+            the order will be descending. The default value is ``-id``. Available fields are:
+
+            - :ref:`id<id>`
+            - :ref:`owner<owner>`
+            - :ref:`source_type<source_type>`
+            - :ref:`koji_event<koji_event>`
+            - :ref:`state<state>`
+            - :ref:`time_to_expire<time_to_expire>`
+            - :ref:`time_done<time_done>`
+            - :ref:`time_removed<time_removed>`
+
+        :statuscode 200: Composes are returned.
+        :statuscode 404: Compose not found.
+        """
         if id is None:
             p_query = filter_composes(request)
 
@@ -117,7 +154,7 @@ class ODCSAPI(MethodView):
         else:
             compose = Compose.query.filter_by(id=id).first()
             if compose:
-                return jsonify(compose.json()), 200
+                return jsonify(compose.json(True)), 200
             else:
                 raise NotFound('No such compose found.')
 
@@ -125,6 +162,15 @@ class ODCSAPI(MethodView):
     @require_scopes('renew-compose')
     @requires_role('allowed_clients')
     def patch(self, id):
+        """ Extends the compose expiration time or regenerates expired compose.
+
+        :query number id: :ref:`ID<id>` of the compose to update/regenerate.
+        :jsonparam number seconds_to_live: Number of seconds before the compoose expires.
+
+        :statuscode 200: Compose updated and returned.
+        :statuscode 401: User is unathorized.
+        :statuscode 404: Compose not found.
+        """
         if request.data:
             data = request.get_json(force=True)
         else:
@@ -145,10 +191,15 @@ class ODCSAPI(MethodView):
             log.error(err)
             raise NotFound(err)
 
+        # Backward compatibility for old composes which don't have
+        # the compose_type set - we treat them as "test" composes
+        # when regenerating them.
+        compose_type = old_compose.compose_type or "test"
+
         raise_if_input_not_allowed(
             source_types=old_compose.source_type, sources=old_compose.source,
             results=old_compose.results, flags=old_compose.flags,
-            arches=old_compose.arches)
+            arches=old_compose.arches, compose_types=compose_type)
 
         has_to_create_a_copy = old_compose.state in (
             COMPOSE_STATES['removed'], COMPOSE_STATES['failed'])
@@ -164,6 +215,13 @@ class ODCSAPI(MethodView):
             # flush and therefore the compose ID won't be set.
             db.session.flush()
             db.session.commit()
+
+            if CELERY_AVAILABLE and conf.celery_broker_url:
+                if compose.source_type == PungiSourceType.PULP:
+                    generate_pulp_compose.delay(compose.id)
+                else:
+                    generate_pungi_compose.delay(compose.id)
+
             return jsonify(compose.json()), 200
         else:
             # Otherwise, just extend expiration to make it usable for longer
@@ -186,6 +244,29 @@ class ODCSAPI(MethodView):
     @require_scopes('new-compose')
     @requires_role('allowed_clients')
     def post(self):
+        """ Creates new ODCS compose request.
+
+        :jsonparam number seconds_to_live: Number of seconds before the compoose expires.
+        :jsonparam list flags: List of :ref:`compose flags<flags>` defined as strings.
+        :jsonparam list arches: List of :ref:`arches<arches>` the compose should be generated for.
+        :jsonparam list multilib_arches: List of :ref:`multilib arches<multilib_arches>`.
+        :jsonparam string multilib_method: List defining the :ref:`multilib method<multilib_method>`.
+        :jsonparam list lookaside_repos: List of :ref:`lookaside_repos<lookaside_repos>`.
+        :jsonparam string label: String defining the :ref:`label<label>`.
+        :jsonparam string compose_type: String defining the :ref:`compose_type<compose_type>`.
+        :jsonparam object source: The JSON object defining the source of compose.
+        :jsonparam string source["type"]: String defining the :ref:`source type<source_type>`.
+        :jsonparam string source["source"]: String defining the :ref:`source<source>`.
+        :jsonparam list source["packages"]: List defining the :ref:`packages<packages>`.
+        :jsonparam list source["builds"]: List defining the :ref:`builds<builds>`.
+        :jsonparam list source["sigkeys"]: List defining the :ref:`sigkeys<sigkeys>`.
+        :jsonparam list source["koji_event"]: Number defining the :ref:`koji_event<koji_event>`.
+        :jsonparam list source["modular_koji_tags"]: List defining the :ref:`modular_koji_tags<modular_koji_tags>`.
+
+        :statuscode 200: Compose request created and returned.
+        :statuscode 401: Request not in valid format.
+        :statuscode 401: User is unathorized.
+        """
         data = request.get_json(force=True)
         if not data:
             raise ValueError('No JSON POST data submitted')
@@ -200,7 +281,7 @@ class ODCSAPI(MethodView):
             log.error(err)
             raise ValueError(err)
 
-        needed_keys = ["type", "source"]
+        needed_keys = ["type"]
         for key in needed_keys:
             if key not in source_data:
                 err = "Missing %s in source configuration, received: %s" % (key, str(source_data))
@@ -215,9 +296,12 @@ class ODCSAPI(MethodView):
 
         source_type = PUNGI_SOURCE_TYPE_NAMES[source_type]
 
-        # Use list(set()) here to remove duplicate sources.
-        source = list(set(source_data["source"].split(" ")))
-        if not source:
+        source = []
+        if "source" in source_data:
+            # Use list(set()) here to remove duplicate sources.
+            source = list(set(source_data["source"].split(" ")))
+
+        if not source and source_type != PungiSourceType.BUILD:
             err = "No source provided for %s" % source_type
             log.error(err)
             raise ValueError(err)
@@ -243,10 +327,15 @@ class ODCSAPI(MethodView):
                     source_name)
         elif source_type == PungiSourceType.MODULE:
             for module_str in source:
-                if len(module_str.split(":")) < 2:
+                nsvc = module_str.split(":")
+                if len(nsvc) < 2:
                     raise ValueError(
                         'Module definition must be in "n:s", "n:s:v" or '
                         '"n:s:v:c" format, but got %s' % module_str)
+                if nsvc[0] in conf.base_module_names:
+                    raise ValueError(
+                        "ODCS currently cannot create compose with base "
+                        "modules, but %s was requested." % nsvc[0])
 
         source = ' '.join(source)
 
@@ -254,9 +343,9 @@ class ODCSAPI(MethodView):
         if "packages" in source_data:
             packages = ' '.join(source_data["packages"])
 
-        if not packages and source_type == PungiSourceType.KOJI_TAG:
-            raise ValueError(
-                '"packages" must be defined for "tag" source_type.')
+        builds = None
+        if "builds" in source_data:
+            builds = ' '.join(source_data["builds"])
 
         sigkeys = ""
         if "sigkeys" in source_data:
@@ -264,6 +353,8 @@ class ODCSAPI(MethodView):
         else:
             sigkeys = ' '.join(conf.sigkeys)
 
+        koji_event = source_data.get('koji_event', None)
+
         flags = 0
         if "flags" in data:
             for name in data["flags"]:
@@ -284,14 +375,61 @@ class ODCSAPI(MethodView):
         else:
             arches = " ".join(conf.arches)
 
+        multilib_arches = ""
+        if "multilib_arches" in data:
+            multilib_arches = " ".join(data["multilib_arches"])
+
+        lookaside_repos = ""
+        if "lookaside_repos" in data:
+            lookaside_repos = " ".join(data["lookaside_repos"])
+
+        multilib_method = MULTILIB_METHODS["none"]
+        if "multilib_method" in data:
+            for name in data["multilib_method"]:
+                if name not in MULTILIB_METHODS:
+                    raise ValueError("Unknown multilib method \"%s\"" % name)
+                multilib_method |= MULTILIB_METHODS[name]
+
+        modular_koji_tags = None
+        if "modular_koji_tags" in source_data:
+            modular_koji_tags = ' '.join(source_data["modular_koji_tags"])
+
+        module_defaults_url = None
+        if "module_defaults_url" in source_data:
+            module_defaults_url = source_data["module_defaults_url"]
+
+        module_defaults_commit = None
+        if "module_defaults_commit" in source_data:
+            module_defaults_commit = source_data["module_defaults_commit"]
+
+        module_defaults = None
+        # The "^" operator is logical XOR.
+        if bool(module_defaults_url) ^ bool(module_defaults_commit):
+            raise ValueError(
+                'The "module_defaults_url" and "module_defaults_commit" '
+                'must be used together.')
+        elif module_defaults_url and module_defaults_commit:
+            module_defaults = "%s %s" % (module_defaults_url, module_defaults_commit)
+
+        label = data.get("label", None)
+        compose_type = data.get("compose_type", "test")
+
         raise_if_input_not_allowed(
             source_types=source_type, sources=source, results=results,
-            flags=flags, arches=arches)
+            flags=flags, arches=arches, compose_types=compose_type)
 
         compose = Compose.create(
             db.session, self._get_compose_owner(), source_type, source,
             results, seconds_to_live,
-            packages, flags, sigkeys, arches)
+            packages, flags, sigkeys, koji_event, arches,
+            multilib_arches=multilib_arches,
+            multilib_method=multilib_method,
+            builds=builds,
+            lookaside_repos=lookaside_repos,
+            modular_koji_tags=modular_koji_tags,
+            module_defaults_url=module_defaults,
+            label=label,
+            compose_type=compose_type)
         db.session.add(compose)
         # Flush is needed, because we use `before_commit` SQLAlchemy event to
         # send message and before_commit can be called before flush and
@@ -299,12 +437,29 @@ class ODCSAPI(MethodView):
         db.session.flush()
         db.session.commit()
 
+        if CELERY_AVAILABLE and conf.celery_broker_url:
+            if source_type == PungiSourceType.PULP:
+                generate_pulp_compose.delay(compose.id)
+            else:
+                generate_pungi_compose.delay(compose.id)
+
         return jsonify(compose.json()), 200
 
     @login_required
     @require_scopes('delete-compose')
     @requires_role('admins')
     def delete(self, id):
+        """ Marks compose as expired to be removed later from ODCS storage.
+        The compose metadata are still stored in the ODCS database, only the
+        composed files stored in ODCS storage are removed.
+
+        :query number id: :ref:`ID<id>` of the compose to delete.
+
+        :statuscode 200: Compose updated and returned.
+        :statuscode 400: Compose is not in done or failed :ref:`state<state>`.
+        :statuscode 401: User is unathorized.
+        :statuscode 404: Compose not found.
+        """
         compose = Compose.query.filter_by(id=id).first()
         if compose:
             # can remove compose that is in state of 'done' or 'failed'
@@ -331,6 +486,19 @@ class ODCSAPI(MethodView):
 
 class AboutAPI(MethodView):
     def get(self):
+        """ Returns information about this ODCS instance in JSON format.
+
+        :resjson string version: The ODCS server version.
+        :resjson string auth_backend: The name of authorization backend this
+            server is configured with. Can be one of following:
+
+            - ``noauth`` - No authorization is required.
+            - ``kerberos`` - Kerberos authorization is required.
+            - ``openidc`` - OpenIDC authorization is required.
+            - ``kerberos_or_ssl`` - Kerberos or SSL authorization is required.
+            - ``ssl`` - SSL authorization is required.
+        :statuscode 200: Compose updated and returned.
+        """
         json = {'version': version}
         config_items = ['auth_backend']
         for item in config_items:
