@@ -27,7 +27,7 @@ from odcs.server.backend import RemoveExpiredComposesThread
 from odcs.server.pungi import PungiSourceType
 from datetime import datetime, timedelta
 
-from .utils import ModelsBaseTest
+from .utils import ModelsBaseTest, AnyStringWith
 
 import os
 import mock
@@ -67,20 +67,44 @@ class TestRemoveExpiredComposesThread(ModelsBaseTest):
             c = db.session.query(Compose).filter(Compose.id == 1).one()
             self.assertEqual(c.state, state)
 
-    def test_a_compose_which_state_is_done_is_removed(self):
+    @patch("os.unlink")
+    @patch("os.path.realpath")
+    def test_a_compose_which_state_is_done_is_removed(self, realpath, unlink):
         """
         Test that we do remove a compose in done state.
         """
+        realpath.return_value = "/odcs-real"
         c = db.session.query(Compose).filter(Compose.id == 1).one()
         c.time_to_expire = datetime.utcnow() - timedelta(seconds=120)
         c.state = COMPOSE_STATES["done"]
+        c.compose_type = "nightly"
+        c.pungi_compose_id = "compose-1-10-2020110.n.0"
         db.session.add(c)
         db.session.commit()
         self.thread.do_work()
         db.session.expunge_all()
         c = db.session.query(Compose).filter(Compose.id == 1).one()
         self.assertEqual(c.state, COMPOSE_STATES["removed"])
-        self.assertEqual(c.state_reason, 'Compose is expired')
+        self.assertEqual(c.state_reason, 'Compose is expired.')
+        unlink.assert_has_calls([
+            mock.call(AnyStringWith("test_composes/nightly/compose-1-10-2020110.n.0")),
+            mock.call(AnyStringWith("test_composes/nightly/latest-compose-1"))])
+
+    def test_a_compose_which_state_is_done_is_removed_keep_state_reason(self):
+        """
+        Test that we do remove a compose in done state.
+        """
+        c = db.session.query(Compose).filter(Compose.id == 1).one()
+        c.time_to_expire = datetime.utcnow() - timedelta(seconds=120)
+        c.state = COMPOSE_STATES["done"]
+        c.state_reason = "Generated successfully."
+        db.session.add(c)
+        db.session.commit()
+        self.thread.do_work()
+        db.session.expunge_all()
+        c = db.session.query(Compose).filter(Compose.id == 1).one()
+        self.assertEqual(c.state, COMPOSE_STATES["removed"])
+        self.assertEqual(c.state_reason, 'Generated successfully.\nCompose is expired.')
 
     def test_does_not_remove_a_compose_which_is_not_expired(self):
         """
@@ -212,3 +236,18 @@ class TestRemoveExpiredComposesThread(ModelsBaseTest):
         self.thread._remove_compose_dir(toplevel_dir)
         unlink.assert_not_called()
         rmtree.assert_called_once()
+
+    @patch("os.unlink")
+    @patch("os.path.realpath")
+    @patch("os.path.exists")
+    @patch("odcs.server.backend.log.warning")
+    def test_remove_compose_rmtree_error(
+            self, log_warning, exists, realpath, unlink):
+        exists.return_value = True
+        toplevel_dir = "/odcs"
+        realpath.return_value = "/odcs-real"
+
+        # This must not raise an exception.
+        self.thread._remove_compose_dir(toplevel_dir)
+        log_warning.assert_called_once_with(
+            AnyStringWith('Cannot remove some files in /odcs-real:'))
