@@ -20,6 +20,7 @@
 #
 # Written by Jan Kaluza <jkaluza@redhat.com>
 
+import six
 import contextlib
 import json
 
@@ -35,7 +36,8 @@ import odcs.server.auth
 
 from odcs.server import conf, db, app, login_manager, version
 from odcs.server.models import Compose, User
-from odcs.common.types import COMPOSE_STATES, COMPOSE_RESULTS, COMPOSE_FLAGS
+from odcs.common.types import (COMPOSE_STATES, COMPOSE_RESULTS, COMPOSE_FLAGS,
+                               MULTILIB_METHODS)
 from odcs.server.pungi import PungiSourceType
 from .utils import ModelsBaseTest
 from odcs.server.api_utils import validate_json_data
@@ -103,6 +105,9 @@ class ViewBaseTest(ModelsBaseTest):
                 'composer': {},
                 'dev2': {
                     'source_types': ['module']
+                },
+                'dev3': {
+                    'source_types': ['raw_config']
                 }
             },
             'users': {
@@ -110,7 +115,11 @@ class ViewBaseTest(ModelsBaseTest):
                     'arches': ['ppc64', 's390', 'x86_64']
                 },
                 'dev2': {
-                    'source_types': ['module', 'raw_config']
+                    'source_types': ['module', 'raw_config'],
+                    'compose_types': ["test", "nightly"]
+                },
+                'dev3': {
+                    'source_types': ['tag']
                 }
             }
         }
@@ -290,6 +299,7 @@ class TestViews(ViewBaseTest):
                          'result_repo': 'http://localhost/odcs/latest-odcs-%d-1/compose/Temporary' % data['id'],
                          'result_repofile': 'http://localhost/odcs/latest-odcs-%d-1/compose/Temporary/odcs-%d.repo' % (data['id'], data['id']),
                          'time_submitted': data["time_submitted"], 'id': data['id'],
+                         'time_started': None,
                          'time_removed': None,
                          'removed_by': None,
                          'time_to_expire': data["time_to_expire"],
@@ -299,7 +309,16 @@ class TestViews(ViewBaseTest):
                          'koji_event': None,
                          'koji_task_id': None,
                          'packages': None,
-                         'arches': 'x86_64'}
+                         'builds': None,
+                         'arches': 'x86_64',
+                         'multilib_arches': '',
+                         'multilib_method': 0,
+                         'lookaside_repos': '',
+                         'modular_koji_tags': None,
+                         'module_defaults_url': None,
+                         'label': None,
+                         'compose_type': 'test',
+                         'pungi_compose_id': None}
         self.assertEqual(data, expected_json)
 
         db.session.expire_all()
@@ -317,9 +336,12 @@ class TestViews(ViewBaseTest):
                  'flags': ['no_deps']}))
             data = json.loads(rv.get_data(as_text=True))
 
-        self.assertEqual(
-            data['message'],
-            '"packages" must be defined for "tag" source_type.')
+        self.assertEqual(data["state_name"], "wait")
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 1).one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+        self.assertEqual(c.packages, None)
 
     def test_submit_build_nodeps(self):
         with self.test_request_context(user='dev'):
@@ -377,6 +399,23 @@ class TestViews(ViewBaseTest):
             c.results,
             COMPOSE_RESULTS["boot.iso"] | COMPOSE_RESULTS["repository"])
 
+    def test_submit_build_with_koji_event(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26', 'packages': ['ed'],
+                            'koji_event': 123456}}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['koji_event'], 123456)
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 3).one()
+        self.assertEqual(c.koji_event, 123456)
+
     def test_submit_build_sigkeys(self):
         with self.test_request_context(user='dev'):
             flask.g.oidc_scopes = [
@@ -429,6 +468,106 @@ class TestViews(ViewBaseTest):
         c = db.session.query(Compose).filter(Compose.id == 1).one()
         self.assertEqual(c.state, COMPOSE_STATES["wait"])
 
+    def test_submit_build_multilib_arches(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26', 'packages': ['ed']},
+                 'arches': ["ppc64", "s390"], 'multilib_arches': ["x86_64", "ppc64le"]}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['multilib_arches'], 'x86_64 ppc64le')
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 1).one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+
+    def test_submit_build_multilib_method(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26', 'packages': ['ed']},
+                 'arches': ["ppc64", "s390"], 'multilib_method': ["runtime", "devel"]}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['multilib_method'],
+                         MULTILIB_METHODS["runtime"] | MULTILIB_METHODS["devel"])
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 1).one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+
+    def test_submit_build_multilib_method_unknown(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26', 'packages': ['ed']},
+                 'arches': ["ppc64", "s390"], 'multilib_method': ["foo", "devel"]}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(
+            data['message'], 'Unknown multilib method "foo"')
+
+    def test_submit_build_modular_koji_tags(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26',
+                            'modular_koji_tags': ['f26-modules']}}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['modular_koji_tags'], "f26-modules")
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 1).one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+
+    def test_submit_build_module_defaults_url(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26',
+                            'module_defaults_url': 'git://localhost.tld/x.git',
+                            'module_defaults_commit': 'master'}}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['module_defaults_url'], 'git://localhost.tld/x.git master')
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 1).one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+
+    def test_submit_build_module_defaults_url_no_branch(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26',
+                            'module_defaults_url': 'git://localhost.tld/x.git'}}))
+            data = json.loads(rv.get_data(as_text=True))
+            self.assertEqual(data['status'], 400)
+            self.assertEqual(data['error'], 'Bad Request')
+            self.assertEqual(data['message'],
+                             'The "module_defaults_url" and "module_defaults_commit" '
+                             'must be used together.')
+
     def test_submit_build_duplicate_sources(self):
         with self.test_request_context(user='dev'):
             flask.g.oidc_scopes = [
@@ -446,6 +585,43 @@ class TestViews(ViewBaseTest):
         c = db.session.query(Compose).filter(Compose.id == 1).one()
         self.assertEqual(c.state, COMPOSE_STATES["wait"])
 
+    def test_submit_build_extra_builds(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'f26', 'packages': ['ed'],
+                            'builds': ['foo-1-1', 'bar-1-1']}}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['builds'], 'foo-1-1 bar-1-1')
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 3).one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+        self.assertEqual(c.builds, 'foo-1-1 bar-1-1')
+
+    def test_submit_build_source_type_build(self):
+        with self.test_request_context(user='dev'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'build', 'packages': ['ed'],
+                            'builds': ['foo-1-1', 'bar-1-1']}}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(data['builds'], 'foo-1-1 bar-1-1')
+
+        db.session.expire_all()
+        c = db.session.query(Compose).filter(Compose.id == 3).one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+        self.assertEqual(c.source_type, PungiSourceType.BUILD)
+        self.assertEqual(c.builds, 'foo-1-1 bar-1-1')
+
     def test_submit_build_resurrection_removed(self):
         self.c1.state = COMPOSE_STATES["removed"]
         self.c1.reused_id = 1
@@ -522,7 +698,24 @@ class TestViews(ViewBaseTest):
 
         self.assertEqual(
             data['message'],
-            'User dev not allowed to operate with compose with source_types=repo')
+            'User dev not allowed to operate with compose with source_types=repo.')
+
+    @patch.object(odcs.server.config.Config, 'raw_config_urls',
+                  new={"pungi_cfg": "http://localhost/pungi.conf#%s"})
+    def test_submit_build_not_allowed_compose_type(self):
+        with self.test_request_context(user='dev2'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'raw_config', 'source': 'pungi_cfg#hash'},
+                 'compose_type': 'production'}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(
+            data['message'],
+            'User dev2 not allowed to operate with compose with compose_types=production.')
 
     def test_submit_build_unknown_source_type(self):
         with self.test_request_context(user='dev'):
@@ -551,6 +744,20 @@ class TestViews(ViewBaseTest):
             data["message"], 'Module definition must be in "n:s", "n:s:v" or '
             '"n:s:v:c" format, but got x')
 
+    def test_submit_module_build_base_module_in_source(self):
+        with self.test_request_context(user='dev2'):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            rv = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'module', 'source': 'testmodule:master platform:x'}}))
+            data = json.loads(rv.get_data(as_text=True))
+
+        self.assertEqual(
+            data["message"], 'ODCS currently cannot create compose with base '
+            'modules, but platform was requested.')
+
     def test_submit_build_per_user_source_type_allowed(self):
         with self.test_request_context(user='dev2'):
             flask.g.oidc_scopes = [
@@ -576,7 +783,7 @@ class TestViews(ViewBaseTest):
 
         self.assertEqual(
             data['message'],
-            'User dev2 not allowed to operate with compose with source_types=tag')
+            'User dev2 not allowed to operate with compose with source_types=tag.')
 
     def test_submit_build_per_group_source_type_allowed(self):
         with self.test_request_context(user="unknown", groups=['dev2', "x"]):
@@ -603,13 +810,14 @@ class TestViews(ViewBaseTest):
 
         self.assertEqual(
             data['message'],
-            'User unknown not allowed to operate with compose with source_types=tag')
+            'User unknown not allowed to operate with compose with source_types=tag.')
 
     def test_query_compose(self):
         resp = self.client.get('/api/1/composes/1')
         data = json.loads(resp.get_data(as_text=True))
         self.assertEqual(data['id'], 1)
         self.assertEqual(data['source'], "testmodule:master")
+        self.assertEqual(data['pungi_config_dump'], None)
 
     def test_query_composes(self):
         resp = self.client.get('/api/1/composes/')
@@ -621,6 +829,7 @@ class TestViews(ViewBaseTest):
         evs = json.loads(resp.get_data(as_text=True))['items']
         self.assertEqual(len(evs), 1)
         self.assertEqual(evs[0]['source'], 'f26')
+        self.assertTrue("pungi_config_dump" not in evs[0])
 
     def test_query_compose_state_done(self):
         resp = self.client.get(
@@ -723,8 +932,8 @@ class TestViews(ViewBaseTest):
 
                 self.assertEqual(resp.status, '400 BAD REQUEST')
                 self.assertEqual(data['status'], 400)
-                self.assertRegexpMatches(data['message'],
-                                         r"Compose \(id=%s\) can not be removed, its state need to be in .*." % new_c.id)
+                six.assertRegex(self, data['message'],
+                                r"Compose \(id=%s\) can not be removed, its state need to be in .*." % new_c.id)
                 self.assertEqual(data['error'], 'Bad Request')
 
     def test_delete_non_exist_compose(self):
@@ -785,6 +994,38 @@ class TestViews(ViewBaseTest):
         c = db.session.query(Compose).filter(Compose.source == 'testmodule:rawhide').one()
         self.assertEqual(c.state, COMPOSE_STATES["wait"])
 
+    def test_can_create_compose_with_user_in_multiple_groups(self):
+        with self.test_request_context(user='another_user', groups=['dev3', 'dev2']):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            resp = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'module', 'source': 'testmodule:rawhide'}}))
+        db.session.expire_all()
+
+        self.assertEqual(resp.status, '200 OK')
+        self.assertEqual(resp.status_code, 200)
+        c = db.session.query(Compose).filter(Compose.source == 'testmodule:rawhide').one()
+        self.assertEqual(c.state, COMPOSE_STATES["wait"])
+
+    def test_cannot_create_compose_with_user_in_multiple_groups(self):
+        with self.test_request_context(user='another_user', groups=['dev3', 'dev2']):
+            flask.g.oidc_scopes = [
+                '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+            ]
+
+            resp = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'tag', 'source': 'testmodule:rawhide'}}))
+            data = json.loads(resp.get_data(as_text=True))
+        db.session.expire_all()
+
+        self.assertEqual(resp.status, '403 FORBIDDEN')
+        self.assertEqual(resp.status_code, 403)
+        self.assertEqual(
+            data['message'],
+            'User another_user not allowed to operate with compose with source_types=tag.')
+
     def test_can_delete_compose_with_user_in_configured_groups(self):
         c3 = Compose.create(
             db.session, "unknown", PungiSourceType.MODULE, "testmodule:testbranch",
@@ -804,35 +1045,51 @@ class TestViews(ViewBaseTest):
         self.assertEqual(resp.status, '202 ACCEPTED')
         self.assertEqual(resp.status_code, 202)
         self.assertEqual(data['status'], 202)
-        self.assertRegexpMatches(data['message'],
-                                 r"The delete request for compose \(id=%s\) has been accepted and will be processed by backend later." % c3.id)
+        six.assertRegex(self, data['message'],
+                        r"The delete request for compose \(id=%s\) has been accepted and will be processed by backend later." % c3.id)
 
-    @patch.object(odcs.server.config.Config, 'max_seconds_to_live', new_callable=PropertyMock)
-    @patch.object(odcs.server.config.Config, 'seconds_to_live', new_callable=PropertyMock)
-    def test_use_seconds_to_live_in_request(self, mock_seconds_to_live, mock_max_seconds_to_live):
-        # if we have 'seconds-to-live' in request < conf.max_seconds_to_live
-        # the value from request will be used
-        mock_seconds_to_live.return_value = 60 * 60 * 24
-        mock_max_seconds_to_live.return_value = 60 * 60 * 24 * 3
-
-        with self.test_request_context(user='dev'):
+    def test_can_create_compose_with_permission_overriden_by_username(self):
+        with self.test_request_context(user='dev3', groups=['dev2']):
             flask.g.oidc_scopes = [
                 '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
             ]
 
-            rv = self.client.post('/api/1/composes/', data=json.dumps(
-                {'source': {'type': 'module', 'source': 'testmodule:master'}, 'seconds-to-live': 60 * 60 * 12}))
-            data = json.loads(rv.get_data(as_text=True))
+            resp = self.client.post('/api/1/composes/', data=json.dumps(
+                {'source': {'type': 'module', 'source': 'testmodule:rawhide'}}))
+        db.session.expire_all()
 
-        time_submitted = datetime.strptime(data['time_submitted'], "%Y-%m-%dT%H:%M:%SZ")
-        time_to_expire = datetime.strptime(data['time_to_expire'], "%Y-%m-%dT%H:%M:%SZ")
-        delta = timedelta(hours=12)
-        self.assertEqual(time_to_expire - time_submitted, delta)
+        self.assertEqual(resp.status, '403 FORBIDDEN')
+        self.assertEqual(resp.status_code, 403)
+
+    @patch.object(odcs.server.config.Config, 'max_seconds_to_live', new_callable=PropertyMock)
+    @patch.object(odcs.server.config.Config, 'seconds_to_live', new_callable=PropertyMock)
+    def test_use_seconds_to_live_in_request(self, mock_seconds_to_live, mock_max_seconds_to_live):
+        # Test that seconds-to-live is still supported to keep backward compatibility.
+        for seconds_to_live in ["seconds-to-live", "seconds_to_live"]:
+            # if we have 'seconds_to_live' in request < conf.max_seconds_to_live
+            # the value from request will be used
+            mock_seconds_to_live.return_value = 60 * 60 * 24
+            mock_max_seconds_to_live.return_value = 60 * 60 * 24 * 3
+
+            with self.test_request_context(user='dev'):
+                flask.g.oidc_scopes = [
+                    '{0}{1}'.format(conf.oidc_base_namespace, 'new-compose')
+                ]
+
+                rv = self.client.post('/api/1/composes/', data=json.dumps(
+                    {'source': {'type': 'module', 'source': 'testmodule:master'},
+                     'seconds_to_live': 60 * 60 * 12}))
+                data = json.loads(rv.get_data(as_text=True))
+
+            time_submitted = datetime.strptime(data['time_submitted'], "%Y-%m-%dT%H:%M:%SZ")
+            time_to_expire = datetime.strptime(data['time_to_expire'], "%Y-%m-%dT%H:%M:%SZ")
+            delta = timedelta(hours=12)
+            self.assertEqual(time_to_expire - time_submitted, delta)
 
     @patch.object(odcs.server.config.Config, 'max_seconds_to_live', new_callable=PropertyMock)
     @patch.object(odcs.server.config.Config, 'seconds_to_live', new_callable=PropertyMock)
     def test_use_max_seconds_to_live_in_conf(self, mock_seconds_to_live, mock_max_seconds_to_live):
-        # if we have 'seconds-to-live' in request > conf.max_seconds_to_live
+        # if we have 'seconds_to_live' in request > conf.max_seconds_to_live
         # conf.max_seconds_to_live will be used
         mock_seconds_to_live.return_value = 60 * 60 * 24
         mock_max_seconds_to_live.return_value = 60 * 60 * 24 * 3
@@ -843,7 +1100,7 @@ class TestViews(ViewBaseTest):
             ]
 
             rv = self.client.post('/api/1/composes/', data=json.dumps(
-                {'source': {'type': 'module', 'source': 'testmodule:master'}, 'seconds-to-live': 60 * 60 * 24 * 7}))
+                {'source': {'type': 'module', 'source': 'testmodule:master'}, 'seconds_to_live': 60 * 60 * 24 * 7}))
             data = json.loads(rv.get_data(as_text=True))
 
         time_submitted = datetime.strptime(data['time_submitted'], "%Y-%m-%dT%H:%M:%SZ")
@@ -854,7 +1111,7 @@ class TestViews(ViewBaseTest):
     @patch.object(odcs.server.config.Config, 'max_seconds_to_live', new_callable=PropertyMock)
     @patch.object(odcs.server.config.Config, 'seconds_to_live', new_callable=PropertyMock)
     def test_use_seconds_to_live_in_conf(self, mock_seconds_to_live, mock_max_seconds_to_live):
-        # if we don't have 'seconds-to-live' in request, conf.seconds_to_live will be used
+        # if we don't have 'seconds_to_live' in request, conf.seconds_to_live will be used
         mock_seconds_to_live.return_value = 60 * 60 * 24
         mock_max_seconds_to_live.return_value = 60 * 60 * 24 * 3
 
@@ -889,6 +1146,7 @@ class TestViews(ViewBaseTest):
                          'result_repo': 'http://localhost/odcs/latest-odcs-%d-1/compose/Temporary' % data['id'],
                          'result_repofile': 'http://localhost/odcs/latest-odcs-%d-1/compose/Temporary/odcs-%d.repo' % (data['id'], data['id']),
                          'time_submitted': data["time_submitted"], 'id': data['id'],
+                         'time_started': None,
                          'time_removed': None,
                          'removed_by': None,
                          'time_to_expire': data["time_to_expire"],
@@ -898,7 +1156,16 @@ class TestViews(ViewBaseTest):
                          'koji_event': None,
                          'koji_task_id': None,
                          'packages': None,
-                         'arches': 'x86_64'}
+                         'builds': None,
+                         'arches': 'x86_64',
+                         'multilib_arches': '',
+                         'multilib_method': 0,
+                         'lookaside_repos': '',
+                         'modular_koji_tags': None,
+                         'module_defaults_url': None,
+                         'label': None,
+                         'compose_type': 'test',
+                         'pungi_compose_id': None}
         self.assertEqual(data, expected_json)
 
         db.session.expire_all()
@@ -950,7 +1217,7 @@ class TestExtendExpiration(ViewBaseTest):
     @patch.object(conf, 'auth_backend', new='noauth')
     def test_bad_request_if_seconds_to_live_is_invalid(self):
         post_data = json.dumps({
-            'seconds-to-live': '600s'
+            'seconds_to_live': '600s'
         })
         with self.test_request_context():
             rv = self.client.patch('/api/1/composes/{0}'.format(self.c1.id),
@@ -959,7 +1226,7 @@ class TestExtendExpiration(ViewBaseTest):
 
             self.assertEqual(400, data['status'])
             self.assertEqual('Bad Request', data['error'])
-            self.assertIn('Invalid seconds-to-live specified in request',
+            self.assertIn('Invalid seconds_to_live specified in request',
                           data['message'])
 
     @patch.object(conf, 'auth_backend', new='noauth')
@@ -976,7 +1243,7 @@ class TestExtendExpiration(ViewBaseTest):
     @patch.object(conf, 'oidc_base_namespace', new='http://example.com/')
     def test_fail_if_extend_non_existing_compose(self):
         post_data = json.dumps({
-            'seconds-to-live': 600
+            'seconds_to_live': 600
         })
         with self.test_request_context():
             flask.g.oidc_scopes = ['http://example.com/new-compose',
@@ -992,7 +1259,7 @@ class TestExtendExpiration(ViewBaseTest):
         db.session.commit()
 
         post_data = json.dumps({
-            'seconds-to-live': 600
+            'seconds_to_live': 600
         })
         with self.test_request_context():
             flask.g.oidc_scopes = [
@@ -1017,7 +1284,7 @@ class TestExtendExpiration(ViewBaseTest):
         expected_time_to_expire = fake_utcnow + timedelta(
             seconds=expected_seconds_to_live)
         post_data = json.dumps({
-            'seconds-to-live': expected_seconds_to_live
+            'seconds_to_live': expected_seconds_to_live
         })
 
         with self.test_request_context():
@@ -1164,9 +1431,13 @@ class TestViewsRawConfig(ViewBaseTest):
 
             self.client.post('/api/1/composes/', data=json.dumps(
                 {'source': {'type': 'raw_config',
-                            'source': 'pungi_cfg#hash'}}))
+                            'source': 'pungi_cfg#hash'},
+                 'label': 'Beta-1.2',
+                 'compose_type': 'nightly'}))
         db.session.expire_all()
         c = db.session.query(Compose).filter(Compose.id == 1).one()
         self.assertEqual(c.state, COMPOSE_STATES["wait"])
         self.assertEqual(c.source_type, PungiSourceType.RAW_CONFIG)
         self.assertEqual(c.source, 'pungi_cfg#hash')
+        self.assertEqual(c.label, 'Beta-1.2')
+        self.assertEqual(c.compose_type, 'nightly')
